"""
At this point this is a stub, it support to be replaced with something that works
"""

import logging
import threading
from datetime import datetime, timedelta


import boto3

from algo.config import config

# IMPORTANT: switch to CLoudWatch Agent
# https://docs.aws.amazon.com/AmazonCloudWatch/latest/monitoring/CloudWatch-Agent-open-telemetry.html

log = logging.getLogger(__name__)


REPORT_INTERVAL = timedelta(seconds=30)


class AgregatedReporter(object):
    """ currently works for the single metric to send amount of data messages received from Oanda"""

    def __init__(self, metric_name: str) -> None:
        self.client = boto3.client('cloudwatch')
        self._last_report = datetime.utcnow()
        self._value = 0
        self._metric_name = metric_name
        self._last_thread = None
        self._namespace = 'AlgoDefaultNamespace'

    def set_namespace(self, namespace: str) -> None:
        self._namespace = namespace

    def _report(self):
        if datetime.utcnow() - self._last_report > REPORT_INTERVAL:
            self._send_report_data(self._value)
            self._value = 0
            self._last_report = datetime.utcnow()

    @staticmethod
    def _send_report_async(client, namespace, metric_name, value):
        client.put_metric_data(
            Namespace=namespace,
            MetricData=[
                {
                    'MetricName': metric_name,
                    # 'Dimensions': [
                    #     {
                    #         'Name': 'DefaultDimensionName',
                    #         'Value': 'DefaultDimensionValue'
                    #     }
                    # ],
                    'Value': value,
                    'Unit': 'Count'
                },
            ]
        )

    def _send_report_data(self, value: int):
        if self._last_thread is not None:
            self._last_thread.join()

        log.info(
            'sending counter value',
            extra={
                'value': value,
                'namespace': self._namespace,
                'metric_name': self._metric_name
            }
        )
        self._last_thread = threading.Thread(
            target=self._send_report_async,
            args=(self.client, self._namespace, self._metric_name, value, )
        )
        self._last_thread.start()

    def inc(self):
        self._value += 1
        self._report()


# reporter = AgregatedReporter('NumberOfDataMessages')


def incr_message_counter():
    log.info('increment-message-counter-stub')
    # reporter.set_namespace(config.metric_namespace)
    # reporter.inc()



import unittest

from algo.constant import Symbol

SYMBOL_STR = 'GBP_USD'


class TestSymbolOperations(unittest.TestCase):
    """ not as much as tests, more playground """

    def test_convert_from_str(self):
        self.assertEqual(Symbol(SYMBOL_STR), Symbol.GBP)

    def test_convert_from_str_back_to_str(self):
        self.assertEqual(Symbol(SYMBOL_STR).value, SYMBOL_STR)



import logging
from typing import List

from v20 import Context
from v20.order import MarketOrder
from v20.response import Response
from v20.trade import Trade

# should be from algo.config, but autofromatter keeps me in a hostage
from algo.config import config
from algo.trading import OrderType
from algo.constant import Symbol  # issues with
from algo.util import d
# from http import HttpStatus

log = logging.getLogger(__name__)


class RestApi:
    @staticmethod
    def get_rest_api():
        return Context(
            hostname=config.oanda_rest_host_name,
            token=config.oanda_token,
        )

    def __init__(self, default_instrument: Symbol) -> None:
        self._api = self.get_rest_api()
        # keep as a copy to shorten all API call
        self._account_id = config.oanda_account_id
        self._default_instrument = default_instrument

    def open(self, direction: OrderType):
        order_spec: MarketOrder = MarketOrder(
            type='MARKET',  # MARKET is a default value
            instrument=self._default_instrument.value,
            # positive buy, negative - sell
            units=1 if direction == OrderType.BUY else -1
        )

        response: Response = self._api.order.create(
            accountID=self._account_id,
            order=order_spec
        )
        log.info('open-order-response', extra=d(response))

    def close_all_trades(self):
        for trade in self.get_open_trades():
            log.info('closing trade', extra=d(trade))
            self.close_trade(trade.id)

    def close_trade(self, trade_id):
        response = self._api.trade.close(
            accountID=self._account_id,
            tradeSpecifier=trade_id
        )
        log.info('close-trade-response', extra=d(response))

    def get_open_trades(self) -> List[Trade]:
        response: Response = self._api.trade.list_open(
            accountID=self._account_id
        )
        log.info(
            'get-open-trades-response',
            extra=d(body=response.body, rtype=type(response))
        )
        return response.body.get('trades', [])



from enum import Enum


class Symbol(Enum):
    JPY = 'USD_JPY'
    GBP = 'GBP_USD'
    EUR = 'EUR_USD'
    AUD = 'AUD_USD'
    GOLD = 'XAU_USD'
    BTC = 'BTC_USD'
    CORN = 'CORN_USD'
    WHEAT = 'WHEAT_USD'
    NAS100_USD = 'NAS100_USD'

    @classmethod
    def all_values(cls, glue: str = ','):
        return glue.join(
            map(lambda x: x.value, cls)
        )



import logging
from functools import partial

from algo.constant import Symbol
from algo.data.common import PriceStream
from algo.data.reader import get_data_reader
from algo.strategy.common import (BaseStrategyParams, Plan, Strategy,
                                  TradingResult)
from algo.trading.state import CurrentState

log = logging.getLogger(__name__)


def plan_strategy(strategy: Strategy, price_stream: PriceStream, params: BaseStrategyParams) -> Plan:
    return partial(strategy, price_stream, params)


def run_simulation(plan: Plan) -> TradingResult:
    trades = []
    state = CurrentState()
    for trade in plan(state):
        trades.append(trade)

    total_profit = sum(tr.profit for tr in trades)
    total_expenses = sum(tr.trade_expenses for tr in trades)
    num_of_trades = len(trades)
    log.info('simulation total',
             extra={
                 'trades_len': num_of_trades,
                 'total_profit': total_profit,
                 'sum_of_expenses': total_expenses
             })
    return TradingResult(total_profit, total_expenses, num_of_trades)


def simulate_trading(simulation_date: str, symbol: Symbol, strategy: Strategy, params: BaseStrategyParams):
    """ This method is used by trainer"""
    log.info('start simulation')
    data_reader = get_data_reader()
    data_iterator = data_reader.week_data(simulation_date, symbol)
    planned_strategy = plan_strategy(strategy, data_iterator, params)

    result = run_simulation(planned_strategy)

    log.info('simulation result', extra={'simulation-result': result})
    return result



from typing import Dict, List, Tuple

from .common import StrategyDescriptor
from .moving_averages import MAStrategyParams
from .moving_averages import run_trading as MovingAverageStrategy

# all strategies put together:
# (yes it can be dynamic, I just like it this way)
moving_average_sample: StrategyDescriptor = StrategyDescriptor(
    'sample-moving-average',
    MovingAverageStrategy,
    MAStrategyParams,
    'Simple moving averages'
)

# STRATEGIES_REGISTER: Final[Tuple[StrategyDescriptor]] = ( # Final will get on board at python3.8
STRATEGIES_REGISTER: Tuple[StrategyDescriptor] = (
    moving_average_sample,
)




class Indicator:
    pass


class MAverage(Indicator):
    def __init__(self, period: int) -> None:
        self._data_stack = []
        self._period = int(period)

    def tick(self, new_data: float) -> None:
        self._data_stack.append(new_data)
        if len(self._data_stack) > self._period:
            self._data_stack.pop(0)

    def ready(self):
        """true if there is enough data"""
        return len(self._data_stack) == self._period

    @property
    def value(self) -> float:
        return sum(self._data_stack) / len(self._data_stack)

    def __str__(self) -> str:
        return f'<MAverage({self._period}): {self.value}>'



from typing import Generator, NamedTuple
import json
from datetime import datetime

from v20.pricing import ClientPrice
from v20.pricing_common import PriceBucket as OandaPriceBucket
from algo.constant import Symbol

from algo.util import str_to_dt

JsonReadyPricePoint = tuple[str, float, float]


class PricePoint(NamedTuple):
    time: datetime
    ask: float
    bid: float

    def to_json(self) -> str:
        return json.dumps([self.time.isoformat(), self.ask, self.bid])

    def to_json_ready(self) -> JsonReadyPricePoint:
        return (self.time.isoformat(), self.ask, self.bid)

    @classmethod
    def from_json_ready(cls, data: JsonReadyPricePoint):
        return cls(*(datetime.fromisoformat(data[0]), data[1], data[2]))

    @staticmethod
    def is_valid_price_data(data: ClientPrice) -> bool:
        """ sometimes oanda just throws the type in the response ..."""
        return (data.time and data.closeoutAsk and data.closeoutBid) is not None

    @classmethod
    def from_oanda_price(cls, data: ClientPrice):
        return cls(str_to_dt(data.time), data.closeoutAsk, data.closeoutBid)


class PriceBucket(NamedTuple):
    """ naming is following oanda's don't have better ideas"""
    price: float
    liquidity: int

    @classmethod
    def from_oanda_price_bucket(cls, price_bucket: OandaPriceBucket):
        return cls(price_bucket.price, price_bucket.liquidity)

    @property
    def dict(self):
        return dict(price=self.price, liquidity=self.liquidity)


class CompletePricePoint(NamedTuple):
    time: datetime
    instrument: Symbol
    ask: float
    bid: float
    asks: list[PriceBucket]
    bids: list[PriceBucket]

    @staticmethod
    def is_valid_price_data(data: ClientPrice) -> bool:
        """ sometimes oanda just throws the type in the response ..."""
        return (data.time and data.closeoutAsk and data.closeoutBid) is not None

    @classmethod
    def from_oanda_price(cls, data: ClientPrice):
        return cls(
            str_to_dt(data.time),
            Symbol(data.instrument),
            data.closeoutAsk,
            data.closeoutBid,
            [PriceBucket.from_oanda_price_bucket(pb) for pb in data.asks],
            [PriceBucket.from_oanda_price_bucket(pb) for pb in data.bids]
        )

    @property
    def dict(self):
        return dict(
            time=self.time.isoformat(),
            instrument=self.instrument.value,
            ask=self.ask,
            bid=self.bid,
            asks=[d.dict for d in self.asks],
            bids=[d.dict for d in self.bids]
        )


PriceStream = Generator[PricePoint, None, None]
CompletePriceStream = Generator[CompletePricePoint, None, None]



import unittest

from v20.pricing import ClientPrice
from v20.pricing_common import PriceBucket as OandaPriceBucket

from algo.constant import Symbol
from algo.data.common import PriceBucket, PricePoint, CompletePricePoint
from algo.util import str_to_dt

STATIC_TIME_STR = "2022-08-26T20:58:01.403692247Z"
STATIC_TIME = str_to_dt(STATIC_TIME_STR)
STATIC_PRICE_ASK = 137.558
STATIC_PRICE_BID = 137.497

INSTRUMENT_STR = 'GBP_USD'
INSTRUMENT_SYMBOL = Symbol(INSTRUMENT_STR)

ASK_PRICE_BUCKETS_DATA = [
    {
        "price": 137.553,
        "liquidity": 1000000
    }, {
        "price": 137.555,
        "liquidity": 2000000
    }]

ASK_BUCKETS = [OandaPriceBucket(**data) for data in ASK_PRICE_BUCKETS_DATA]

BID_PRICE_BUCKETS_DATA = [
    {
        "price": 137.502,
        "liquidity": 1000000
    }, {
        "price": 137.5,
        "liquidity": 2000000
    }, {
        "price": 137.499,
        "liquidity": 2000000
    }
]

BID_BUCKETS = [OandaPriceBucket(**data) for data in BID_PRICE_BUCKETS_DATA]


class TestPricePoint(unittest.TestCase):

    def test_is_valid_price_data(self):
        self.assertTrue(
            PricePoint.is_valid_price_data(
                ClientPrice(
                    time=STATIC_TIME_STR,
                    closeoutBid=STATIC_PRICE_BID,
                    closeoutAsk=STATIC_PRICE_ASK)
            )
        )
        self.assertFalse(PricePoint.is_valid_price_data(
            ClientPrice()
        ))

    def test_from_oanda_price(self):
        self.assertEqual(
            PricePoint.from_oanda_price(
                ClientPrice(
                    time=STATIC_TIME_STR,
                    closeoutBid=STATIC_PRICE_BID,
                    closeoutAsk=STATIC_PRICE_ASK)
            ),
            PricePoint(STATIC_TIME, STATIC_PRICE_ASK, STATIC_PRICE_BID)
        )


class TestCompletePricePoint(unittest.TestCase):

    def test_is_valid_price_data(self):
        self.assertTrue(
            CompletePricePoint.is_valid_price_data(
                ClientPrice(
                    time=STATIC_TIME_STR,
                    closeoutBid=STATIC_PRICE_BID,
                    closeoutAsk=STATIC_PRICE_ASK)
            )
        )
        self.assertFalse(CompletePricePoint.is_valid_price_data(
            ClientPrice()
        ))

    def test_from_oanda_price(self):
        def _price_bucket_from_dict(data: dict) -> PriceBucket:
            return PriceBucket(data['price'], data['liquidity'])

        self.assertEqual(
            CompletePricePoint.from_oanda_price(
                ClientPrice(
                    time=STATIC_TIME_STR,
                    instrument=INSTRUMENT_STR,
                    closeoutBid=STATIC_PRICE_BID,
                    closeoutAsk=STATIC_PRICE_ASK,
                    asks=ASK_BUCKETS,
                    bids=BID_BUCKETS
                )
            ),
            CompletePricePoint(
                STATIC_TIME, INSTRUMENT_SYMBOL, STATIC_PRICE_ASK, STATIC_PRICE_BID, [
                    _price_bucket_from_dict(d) for d in ASK_PRICE_BUCKETS_DATA
                ],
                [
                    _price_bucket_from_dict(d) for d in BID_PRICE_BUCKETS_DATA
                ]
            )
        )


class TestPriceBucket(unittest.TestCase):
    def test_test_case_can_differ(self):
        self.assertNotEqual(
            PriceBucket(11.2, 1000),
            PriceBucket(22.2, 2000),
        )

    def test_kwargs_work_as_expected(self):
        self.assertEqual(
            PriceBucket(11.2, 1000),
            PriceBucket(liquidity=1000, price=11.2),
        )



import logging
from datetime import datetime, timedelta

from influxdb.resultset import ResultSet

from algo.config import config
from algo.constant import Symbol
from algo.util import d, dt_to_rfc3339
from algo.data.cache import cached
from algo.data.common import PricePoint, PriceStream
from algo.data.connection import DataConnection

log = logging.getLogger(__name__)


class DataReader(DataConnection):
    def __init__(self, url, token, org):
        super().__init__(url, token, org)
        self._query_api = self._client.query_api()

    def hour_data(self, date_str: str, symbol: Symbol) -> PriceStream:
        start = datetime.fromisoformat(date_str)
        end = start + timedelta(hours=1)
        yield from cached(
            f'hour-data-{symbol.value}-{start.isoformat()}',
            self.get_data_stream(start, end, symbol)
        )

    def day_data(self, date_str: str, symbol: Symbol) -> PriceStream:
        start = datetime.fromisoformat(date_str)
        end = start + timedelta(days=1)
        yield from cached(
            f'day-data-{symbol.value}-{start.isoformat()}',
            self.get_data_stream(start, end, symbol)
        )

    def week_data_direct(self, date_str: str, symbol: Symbol) -> PriceStream:
        week_dt = datetime.fromisoformat(date_str)
        start = week_dt - timedelta(days=week_dt.weekday())
        end = start + timedelta(days=6)
        yield from cached(
            f'week-data-{symbol.value}-{start.isoformat()}',
            self.get_data_stream(start, end, symbol)
        )

    def week_data(self, date_str: str, symbol: Symbol) -> PriceStream:
        week_dt = datetime.fromisoformat(date_str)
        start = week_dt - timedelta(days=week_dt.weekday())
        end = start + timedelta(days=6)
        # 1 weeek data set is too big to cache at this point so doing it iteratively
        current_day = start
        while current_day <= end:
            log.info(
                'getting data for the week by the day',
                extra=d(current_day)
            )
            yield from self.day_data(current_day.isoformat(), symbol)
            current_day += timedelta(days=1)

    def get_result_set(self, symbol: Symbol, start: datetime, end: datetime) -> ResultSet:
        query_str = f"""
            from(bucket: "tsdata")
              |> range(start: {dt_to_rfc3339(start)}, stop: {dt_to_rfc3339(end)})
              |> filter(fn: (r) => r["instrument"] == "{symbol.value}")
              |> filter(fn: (r) => r["_measurement"] == "price")
              |> pivot(rowKey: ["_time"], columnKey: ["_field"], valueColumn: "_value")
        """
        return self._query_api.query(query_str)

    def get_data_stream(
            self, start: datetime, end: datetime, symbol: Symbol
    ) -> PriceStream:
        log.debug('getting chunk starting', extra=d(start))
        results = self.get_result_set(symbol, start, end)
        log.info('query-result', extra={'r': results})
        for flux_table in results:
            for record in flux_table:
                yield PricePoint(record['_time'], record['ask'], record['bid'])

    def get_data_stream_iterative_ready(
            self, start: datetime, end: datetime, symbol: Symbol
    ) -> PriceStream:
        """
        not in use consider removal
        """
        current_start = start
        keep_loading = True
        while keep_loading and current_start < end:
            log.debug('getting next chunk %s', current_start)
            results = self.get_result_set(symbol, current_start, end)
            log.info('query-result', extra={'r': results})
            for flux_table in results:
                for record in flux_table:
                    # following extra condition is present due to milliseconds round up of the influx
                    keep_loading = current_start <= record['_time']
                    if keep_loading:
                        current_start = record['_time']
                        log.info('current_start', extra=d(start=current_start))
                        yield PricePoint(record['_time'], record['ask'], record['bid'])


def get_data_reader() -> DataReader:
    return DataReader(
        config.influx_url,
        config.influx_token,
        config.influx_org
    )



import logging
from datetime import datetime, timedelta
from typing import List
from operator import itemgetter
import pytz

from influxdb_client.client.write_api import SYNCHRONOUS
import boto3
from botocore.exceptions import ClientError
import ujson

from algo.config import config
from algo.data.connection import DataConnection
from algo.data.common import CompletePricePoint

log = logging.getLogger(__name__)


class InfluxDataReporter(DataConnection):  # pylint: disable=too-few-public-methods
    def __init__(self, url, token, org):
        super().__init__(url, token, org)
        self._write_api = self._client.write_api(write_options=SYNCHRONOUS)

    def report(self, price: CompletePricePoint):
        json_body = [
            {
                "measurement": 'price',
                "tags": {
                    "instrument": price.instrument.value,
                },
                "time": price.time,
                "fields": {
                    "ask": price.ask,
                    "bid": price.bid,
                }
            }
        ]
        self._write_api.write(bucket='tsdata', record=json_body)
        log.info('data sent')


# this one has a space to lower current limit is 5 requests per second
DATA_SENDING_INTERVAL = timedelta(milliseconds=5000)
ALREADY_EXISTS_ERROR_CODE = 'ResourceAlreadyExistsException'


class CloudWatchDataReporter:

    def _init_connection(self, log_group_name, log_stream_name):
        api = boto3.client('logs')

        try:
            api.create_log_group(logGroupName=log_group_name)
        except ClientError as exc:
            log.info(
                'error creating log group',
                extra={
                    'e': exc, 't': type(exc),
                    'error-details': exc.response
                }
            )
            if exc.response['Error']['Code'] != ALREADY_EXISTS_ERROR_CODE:
                raise

        try:
            api.create_log_stream(
                logGroupName=log_group_name,
                logStreamName=log_stream_name
            )
        except ClientError as exc:
            log.error(
                'error creating log stream',
                extra={
                    'e': exc, 't': type(exc),
                    'error-details': exc.response
                }
            )
            if exc.response['Error']['Code'] != ALREADY_EXISTS_ERROR_CODE:
                raise
        return api

    def __init__(self, log_group_name, log_stream_name) -> None:
        self._log_group_name = log_group_name
        self._log_stream_name = log_stream_name

        self._price_stack: List[CompletePricePoint] = []
        self._last_report_time = datetime.now()
        self._last_sequence_token = None
        self._api = self._init_connection(log_group_name, log_stream_name)

    def report(self, price: CompletePricePoint):
        self._price_stack.append(price)
        self._send_data()

    def _init_sequence_token(self):
        if self._last_sequence_token is None:
            streams = self._api.describe_log_streams(
                logGroupName=self._log_group_name, logStreamNamePrefix=self._log_stream_name
            )
            # uploadSequenceToken will be absent on first run for new stream
            self._last_sequence_token = streams['logStreams'][0].get(
                'uploadSequenceToken'
            )

    def _send_data(self):
        self._init_sequence_token()

        # TBD: need to catch potential errors and reconnect
        if datetime.now() - self._last_report_time > DATA_SENDING_INTERVAL:
            log.info(
                'time-since-last-report',
                extra={
                    'dt': datetime.now() - self._last_report_time,
                    'data-count': len(self._price_stack)
                }
            )
            self._last_report_time = datetime.now()
            records_batch = [{
                'timestamp': int(p.time.astimezone(pytz.utc).timestamp() * 1000),
                'message': ujson.dumps(p.dict)
            } for p in self._price_stack]

            log_events_args = {
                'logGroupName': self._log_group_name,
                'logStreamName': self._log_stream_name,
                'logEvents': sorted(records_batch, key=itemgetter('timestamp'), reverse=False),
            }
            self._price_stack = []

            if self._last_sequence_token is not None:
                log_events_args['sequenceToken'] = self._last_sequence_token

            resp = self._api.put_log_events(**log_events_args)
            log.debug('put_log_event_response', extra={'response': resp})
            self._last_sequence_token = resp.get('nextSequenceToken', None)


def get_influx_data_reporter() -> InfluxDataReporter:
    return InfluxDataReporter(
        config.influx_url,
        config.influx_token,
        config.influx_org
    )


def get_cloud_watch_reporter() -> CloudWatchDataReporter:
    return CloudWatchDataReporter(
        config.aws_cloud_watch_log_group_name,
        config.aws_cloud_watch_log_stream_name
    )



import logging
import os

import yaml

log = logging.getLogger(__name__)


class Config(object):
    config_data = {}

    @classmethod
    def load_config(cls):
        config_filename = cls._from_env('CONFIG_FILE', '~/config.yml')
        log.info(
            'loading config file',
            extra={'config-filename': config_filename}
        )
        config_data = {}
        try:
            with open(config_filename, 'r', encoding='utf-8') as config_file:
                config_data = yaml.full_load(config_file)
                # todo: add some masking to the security configs
                log.debug('config data', extra={'config-data': config_data})

        except FileNotFoundError:
            log.exception('no config file')
        return config_data

    def __init__(self) -> None:
        self.config_data = self.load_config()

    def set_config(self, data: dict):
        self.config_data = data

    # todo: remove with next update

    @property
    def influx_host(self):
        return self._from_env('INFLUX_HOST', 'localhost')

    @staticmethod
    def _from_env(name: str, default=None):
        return os.environ.get(name, default)

    @property
    def redis(self):
        return self.config_data.get('redis', {})

    @property
    def redis_url(self) -> str:
        return self.redis.get('url')

    @property
    def influx(self):
        return self.config_data.get('influx', {})

    @property
    def influx_token(self):
        return self.influx.get('token')

    @property
    def influx_url(self):
        return self.influx.get('url')

    @property
    def influx_org(self):
        return self.influx.get('org')

    @property
    def oanda(self):
        return self.config_data.get('oanda', {})

    @property
    def oanda_token(self):
        return self.oanda.get('TOKEN')

    @property
    def oanda_stream_host_name(self):
        return self.oanda.get('STREAM_HOST_NAME')

    @property
    def oanda_rest_host_name(self):
        return self.oanda.get('REST_HOST_NAME')

    @property
    def oanda_account_id(self):
        return self.oanda.get('ACCOUNT_ID')

    @property
    def metric(self):
        return self.config_data.get('metric', {})

    @property
    def metric_namespace(self):
        return self.metric.get('namespace', 'DefaultMetricNamespace')

    # aws stuff
    @property
    def aws(self):
        return self.config_data.get('aws', {})

    @property
    def aws_cloud_watch(self):
        return self.aws.get('cloud_watch', {})

    @property
    def aws_cloud_watch_log_group_name(self):
        return self.aws_cloud_watch.get('log_group_name', 'default-log-name')

    @property
    def aws_cloud_watch_log_stream_name(self):
        return self.aws_cloud_watch.get('log_stream_name', 'algo-stream-name')


# config = Config()
class LazyConfig():
    def __init__(self) -> None:
        self.config = None

    def __getattr__(self, item):
        if self.config is None:
            self.config = Config()
        return getattr(self.config, item)


config: Config = LazyConfig()



#!/usr/bin/python3

# import json
import logging.config
from algo.data.common import CompletePriceStream

# from algo.config import config
# from algo.constant import Symbol
# from algo.data.reader import PriceStream, get_data_reader
# from algo.simulation import simulate_trading
# from algo.strategy.moving_averages import MAStrategyParams, run_simulation
# from algo.trading import PricePoint
# from algo.util import d, now
# from algo.strategy import moving_average_sample
from algo.data.live import get_live_price_stream, get_complete_prices_generator
from algo.util import d
# from algo.config import config
from algo.data.reporter import get_influx_data_reporter, get_cloud_watch_reporter

logging.config.fileConfig(
    './logging.conf',
    # TBD: figure out why lack of the following line disables logging inside module
    disable_existing_loggers=False
)

log = logging.getLogger(__name__)


# def serialize_data_record(data_record):
#     """
#     input is a FluxRecord
#     """
#     return data_record['result']['_measurement']


# def read_all(data: PriceStream):
#     data_points_counter = 0
#     total_length = 0  # in chars
#     total_ask = 0.0
#     total_bid = 0.0
#     for data_point in data:
#         # log.info('data-point', extra=d(data_point))
#         # log.info('data-point', extra=d(data_point[0].isoformat()))
#         total_ask += data_point[1]
#         total_bid += data_point[2]

#         data_serialized = data_point.to_json()
#         total_length += len(data_serialized)
#         data_points_counter += 1
#     return data_points_counter, total_length, total_ask, total_bid


def try_listener():
    """write the fucking tests"""
    # config.set_config({

    # })
    price_stream: CompletePriceStream = get_live_price_stream(
        price_reader=get_complete_prices_generator
    )
    influx_reporter = get_influx_data_reporter()
    cw_reporter = get_cloud_watch_reporter()
    for price in price_stream:
        log.info('price-point', extra=d(p=price))
        influx_reporter.report(price)
        cw_reporter.report(price)


if __name__ == '__main__':
    log.info('playground home will be here')
    try_listener()
    # price_generator = (
    #     PricePoint(now(), 1, 2),
    #     PricePoint(now(), 2, 3),
    #     PricePoint(now(), 3, 4),
    #     PricePoint(now(), 5, 6),
    # )
    # run_simulation(price_generator, MAStrategyParams(1, 2))
    # data_reader = get_data_reader()

    # data_iterator = data_reader.day_data(
    #     '2022-06-20 00:00:00+00:00', Symbol.JPY)
    # log.info('start-reading')
    # data_stat = read_all(data_iterator)
    # log.info('data-stat', extra=d(data_stat))
    # log.info('first-read-complete')
    # data_iterator = data_reader.day_data(
    #     '2022-06-20 00:00:00+00:00', Symbol.JPY)
    # log.info('start-second-reading')
    # data_stat = read_all(data_iterator)
    # log.info('data-stat', extra=d(data_stat))
    # log.info('second-read-complete')

    # strategy = moving_average_sample
    # result = simulate_trading(
    #     '2022-06-20',
    #     Symbol.JPY,
    #     strategy.module,
    #     strategy.args(2, 3)
    # )
    # log.info('simulation-complete', extra={'result': result})
